﻿using Newtonsoft.Json;
using System;
using System.Net;
using System.Drawing;
using System.IO;
using VLB_RestClient.Composites;
using System.Globalization;
using System.Collections.Generic;
using System.Linq;



namespace VLB_RestClient
{
    /// <summary>
    /// Provides Methods and Enumerations to access the VLB REST-Api.
    /// </summary>
    public class RestRequest
    {
        //Enums
        /// <summary>
        /// Defines the Sort-Field when searching for Titles on vlb.de via the REST-Api.
        /// </summary>
        public enum sortField
        {
            /// <summary>Sort by the VLB Score Value, so more popular Products are shown first / last.</summary>
            score,
            /// <summary>Sort by ISBN.</summary>
            isbn,
            /// <summary>Sort by Title.</summary>
            title,
            /// <summary>Sort by Author.</summary>
            author,
            /// <summary>Sort by Publisher.</summary>
            publisher,
            /// <summary>Sort by German Price.</summary>
            price,
            /// <summary>Sort by Product Type.</summary>
            productType,
            /// <summary>Sort by Publication Date.</summary>
            publicationDate,
            /// <summary>Sort by Creation Date (Date, when the Product was added to the VLB Database).</summary>
            createDate,
            /// <summary>Sort by last Date (Date, when the Product was last updated in the VLB Database).</summary>
            lastChangeDate
        }

        /// <summary>
        /// Defines the Sort-Sequence (ascending or descending) when searching for Titles on vlb.de via the REST-Api.
        /// </summary>
        public enum sortSequence
        {
            /// <summary>Sort the Search Results in ascending Order.</summary>
            asc,
            /// <summary>Sort the Search Results in descending Order.</summary>
            desc
        }

        /// <summary>
        /// Defines the Database to search during Search Requests.
        /// </summary>
        public enum source
        {
            /// <summary>Search the VLB Database (Verzeichnis lieferbarer Bücher).</summary>
            VLB,
            /// <summary>Search the SBZ Database (Schweizer Buchzentrum).</summary>
            SBZ,
            /// <summary>Search the ÖSB Database (Österreichisches Schulbuchzentrum).</summary>
            OESB
        }

        /// <summary>
        /// Defines the Product Status to search for (active, archived or all).
        /// </summary>
        public enum status
        {
            /// <summary>Search for active Products.</summary>
            active,
            /// <summary>Search for archived Products.</summary>
            archived,
            /// <summary>Search for active and archived Products.</summary>
            all
        }

        /// <summary>
        /// Defines the Index to search.
        /// </summary>
        public enum index
        {
            /// <summary>Search the Author Index.</summary>
            author,
            /// <summary>Search the Publisher Index.</summary>
            publisher,
            /// <summary>Search the Title Index.</summary>
            title,
            /// <summary>Search the Keyword Index.</summary>
            keyword,
            /// <summary>Search the Set Index - Set and Collection Index are the same.</summary>
            set,
            /// <summary>Search the Collection Index - Set and Collection Index are the same.</summary>
            collection,
            /// <summary>Search the ISBN Index.</summary>
            identifier
        }

        /// <summary>
        /// Defines the Cover Size when downloading a Cover.
        /// </summary>
        public enum coverSize
        {
            /// <summary>Small Image Size. Height: 90, Width: scaled to Height.</summary>
            s,
            /// <summary>Medium Image Size. Height: 200, Width: scaled to Height.</summary>
            m,
            /// <summary>Large Image Size. Height: scaled to Width, Width: 599.</summary>
            l,
            /// <summary>Image with original Size.</summary>
            original
        }

        //Members
        /// <summary>
        /// Endpoint to the vlb.de-Live-Systems. This is the default Endpoint.
        /// </summary>
        public readonly string endpointLIVE = "http://api.vlb.de/api/v1/";

        /// <summary>
        /// Endpoint to the vlb.de-Test-Systems. This Endpoint may be used to savely test your Program.
        /// </summary>
        public readonly string endpointTEST = "http://test-api.vlb.de/api/v1/";

        /// <summary>
        /// Currently used Endpoint for all Requests. By default, it is set to endpointLIVE. Most of the Time, you will not have to change this Parameter.
        /// </summary>
        public string endpoint = "http://api.vlb.de/api/v1/";



        /// <summary>
        /// Performs a Login with the User's Credentials. Returns a statefull Access Token, which is needed for all further API Requests.
        /// </summary>
        /// <param name="account">A valid VLB Account.</param>
        /// <param name="password">A valid VLB Password.</param>
        /// <returns>A statefull VLB Access Token.</returns>
        public string login(string account, string password)
        {
            //Check Account and Password
            if (string.IsNullOrWhiteSpace(account))
            {
                throw new Exception("Account may not be null or empty!");
            }

            if (string.IsNullOrWhiteSpace(password))
            {
                throw new Exception("Password may not be null or empty!");
            }

            //Variables
            string accessToken = null;

            //Login with Login Data
            RestClient client = new RestClient();
            client.EndPoint = endpoint + "login";
            client.ContentTypeString = "application/json";
            client.Method = HttpVerb.POST;
            client.PostData = "{ \"username\": \"" + account.Trim() + "\", \"password\": \"" + password.Trim() + "\" }";

            //Make Request
            try
            {
                client.MakeRequest("", ref accessToken);

                return accessToken;
            }
            catch (Exception ex)
            {
                throw new Exception("An Error occured:\n\n" + ex.Message);
            }
        }



        /// <summary>
        /// Sends a Logout Request, releasing the current statefull Access Token.
        /// </summary>
        /// <param name="accessToken">The statefull Access Token to release.</param>
        public void logout(string accessToken)
        {
            //Check Token
            if (string.IsNullOrWhiteSpace(accessToken))
            {
                throw new Exception("AccessToken may not be null or empty!");
            }

            //Variables
            RestClient client;

            //Init new RestClient
            client = new RestClient();
            client.EndPoint = endpoint + "logout";
            client.AccessToken = string.Format("Bearer {0}", accessToken);
            client.ContentTypeString = "application/json";
            client.Method = HttpVerb.GET;

            //Send Request
            try
            {
                string notUsed = "";
                client.MakeRequest("", ref notUsed);
            }
            catch (Exception ex)
            {
                throw new Exception("An Error occured:\n\n" + ex.Message);
            }
        }



        /// <summary>
        /// Performs a (boolean) Search and returns the Results as JsonContainer.
        /// </summary>
        /// <param name="accessToken">A statefull Access Token generated by the login-Method or a fixed stateless Access Token.</param>
        /// <param name="searchQuery">A (boolean) Search Qery, which will specify the Titles searched for, e.g. "TI=Nice Books AND PR=10". Please refer the VLB Rest Api Manual for boolean Search Query Examples.</param>
        /// <param name="page">The current Page to show, if more Titles were found than may be displayed on a single Page.</param>
        /// <param name="size">The Amount of Titles found to show on a single Page.</param>
        /// <param name="sortField">The Field to sort the Titles found.</param>
        /// <param name="sortSequence">Sorting Sequence (ascending or descending).</param>
        /// <param name="source">The Source to search for.</param>
        /// <param name="status">Search for active, archived or all Products.</param>
        /// <returns>A JsonDataContainerSearch, containing all Search Results of the current Page.</returns>
        public JsonDataContainerSearch search(string accessToken, string searchQuery, int page = 1, int size = 50, sortField sortField = sortField.score, sortSequence sortSequence = sortSequence.desc, source source = source.VLB, status status = status.all)
        {
            //Check Parameters
            if (string.IsNullOrWhiteSpace(accessToken))
            {
                throw new Exception("AccessToken may not be null or empty!");
            }

            if (string.IsNullOrWhiteSpace(searchQuery))
            {
                throw new Exception("SearchQuery may not be null or empty!");
            }

            if(page <= 0)
            {
                throw new Exception("Page may not be equal or less than 0!");
            }

            if (size <= 0)
            {
                throw new Exception("Size may not be equal or less than 0!");
            }

            //Variables
            RestClient client;
            HttpWebResponse json;
            string responseAsString = null;
            string param_page;
            string param_size;
            string param_sort;
            string param_direction;
            string param_search;
            string param_source;
            string param_status;

            //Set Search Parameters
            param_page = "?page=" + page.ToString();
            param_size = "&size=" + size.ToString();
            param_sort = "&sort=" + sortField.ToString();
            param_direction = "&direction=" + sortSequence.ToString();
            param_search = "&search=" + searchQuery.ToString();
            param_source = "&source=" + source.ToString();
            param_status = status != status.all ? "&status=" + status.ToString() : "";

            //Init new RestClient
            client = new RestClient();
            client.EndPoint = string.Format(endpoint + "products{0}{1}{2}{3}{4}{5}{6}", param_page, param_size, param_sort, param_direction, param_search, param_source, param_status);
            client.AccessToken = string.Format("Bearer {0}", accessToken);
            client.ContentTypeString = "application/json-short";
            client.Method = HttpVerb.GET;

            //Send Request
            try
            {
                json = client.MakeRequest("", ref responseAsString);
            }
            catch (Exception ex)
            {
                throw new Exception("An Error occured:\n\n" + ex.Message);
            }

            //Init Json.net Deserializer
            JsonSerializerSettings jsonSerializerSettings = new JsonSerializerSettings();
            jsonSerializerSettings.MissingMemberHandling = MissingMemberHandling.Ignore;
            jsonSerializerSettings.NullValueHandling = NullValueHandling.Ignore;

            //Parse and return Response
            return JsonConvert.DeserializeObject<JsonDataContainerSearch>(responseAsString, jsonSerializerSettings);
        }



        /// <summary>
        /// Performs a Stack Search with the provided ISBNs and returns the Results as JsonContainer.
        /// </summary>
        /// <param name="accessToken">A statefull Access Token generated by the login-Method or a fixed stateless Access Token.</param>
        /// <param name="ISBNs">A List of ISBNs to search via the Database.</param>
        /// <param name="page">The current Page to show, if more Titles were found than may be displayed on a single Page.</param>
        /// <param name="size">The Amount of Titles found to show on a single Page.</param>
        /// <param name="sortField">The Field to sort the Titles found.</param>
        /// <param name="sortSequence">Sorting Sequence (ascending or descending).</param>
        /// <param name="source">The Source to search for.</param>
        /// <param name="status">Search for active, archived or all Products.</param>
        /// <returns>A JsonDataContainerSearch, containing all Search Results of the current Page.</returns>
        public JsonDataContainerSearch stackSearch(string accessToken, List<string> ISBNs, int page = 1, int size = 50, sortField sortField = sortField.score, sortSequence sortSequence = sortSequence.desc, source source = source.VLB, status status = status.all)
        {
            //Check Parameters
            if (string.IsNullOrWhiteSpace(accessToken))
            {
                throw new Exception("AccessToken may not be null or empty!");
            }

            for(int i = 0; i < ISBNs.Count; i++)
            {
                ISBNs[i] = ISBNs[i].Replace("-", "");

                if(!ISBNs[i].All(Char.IsDigit))
                {
                    throw new Exception("ISBN-List may not contain non-digit Characters!");
                }

                if(ISBNs[i].Length != 13)
                {
                    throw new Exception("ISBN-List Items must be 13 Digits long!");
                }
            }

            if (page <= 0)
            {
                throw new Exception("Page may not be equal or less than 0!");
            }

            if (size <= 0)
            {
                throw new Exception("Size may not be equal or less than 0!");
            }

            //Variables
            RestClient client;
            HttpWebResponse json;
            string responseAsString = null;
            string param_page;
            string param_size;
            string param_sort;
            string param_direction;
            string param_source;
            string param_status;
            string param_postData;

            //Set Search Parameters
            param_page = "?page=" + page.ToString();
            param_size = "&size=" + size.ToString();
            param_sort = "&sort=" + sortField.ToString();
            param_direction = "&direction=" + sortSequence.ToString();
            param_source = "&source=" + source.ToString();
            param_status = status != status.all ? "&status=" + status.ToString() : "";

            param_postData = "{ \"content\": [ ";

            for (int i = 0; i < ISBNs.Count; i++)
            {
                ISBNs[i] = "{ \"isbn\": \"" + ISBNs[i] + "\" }";
            }

            param_postData += string.Join(",", ISBNs.ToArray<string>());
            param_postData += " ] }";

            //Init new RestClient
            client = new RestClient();
            client.EndPoint = string.Format(endpoint + "products{0}{1}{2}{3}{4}{5}", param_page, param_size, param_sort, param_direction, param_source, param_status);
            client.AccessToken = string.Format("Bearer {0}", accessToken);
            client.ContentTypeString = "application/json-short";
            client.Method = HttpVerb.POST;
            client.PostData = param_postData;

            //Send Request
            try
            {
                json = client.MakeRequest("", ref responseAsString);
            }
            catch (Exception ex)
            {
                throw new Exception("An Error occured:\n\n" + ex.Message);
            }

            //Init Json.net Deserializer
            JsonSerializerSettings jsonSerializerSettings = new JsonSerializerSettings();
            jsonSerializerSettings.MissingMemberHandling = MissingMemberHandling.Ignore;
            jsonSerializerSettings.NullValueHandling = NullValueHandling.Ignore;

            //Parse and return Response
            return JsonConvert.DeserializeObject<JsonDataContainerSearch>(responseAsString, jsonSerializerSettings);
        }



        /// <summary>
        /// Requests Title Detail Data for a specified Product.
        /// </summary>
        /// <param name="accessToken">A statefull Access Token generated by the login-Method or a fixed stateless Access Token.</param>
        /// <param name="isbn">The unique 13-digit Identifier, with or without Hyphen, e.g. "123-4-56789-0123" or "1234567890123".</param>
        /// <returns>A JsonDataContainerDetail, containing the Detail Data for the specified Title.</returns>
        public JsonDataContainerDetail getTitleDetailData(string accessToken, string isbn)
        {
            //Check Parameters
            if (string.IsNullOrWhiteSpace(accessToken))
            {
                throw new Exception("AccessToken may not be null or empty!");
            }

            if (string.IsNullOrWhiteSpace(isbn))
            {
                throw new Exception("ISBN may not be null or empty!");
            }

            //Variables
            RestClient client;
            HttpWebResponse json;
            string responseAsString = null;

            //Init new RestClient
            client = new RestClient();
            client.EndPoint = endpoint + "product/" + isbn.Replace("-", "").Trim() + @"/isbn13";
            client.AccessToken = string.Format("Bearer {0}", accessToken);
            client.ContentTypeString = "application/json";
            client.Method = HttpVerb.GET;

            //Send Request
            try
            {
                json = client.MakeRequest("", ref responseAsString);
            }
            catch (Exception ex)
            {
                throw new Exception("An Error occured:\n\n" + ex.Message);
            }

            //Init Json.net Deserializer
            JsonSerializerSettings jsonSerializerSettings = new JsonSerializerSettings();
            jsonSerializerSettings.MissingMemberHandling = MissingMemberHandling.Ignore;
            jsonSerializerSettings.NullValueHandling = NullValueHandling.Ignore;

            //Parse and return Response
            return JsonConvert.DeserializeObject<JsonDataContainerDetail>(responseAsString, jsonSerializerSettings);
        }



        /// <summary>
        /// Requests the Publisher Data for the specified mvbId.
        /// </summary>
        /// <param name="accessToken">A statefull Access Token generated by the login-Method or a fixed stateless Access Token.</param>
        /// <param name="mvbId">A unique mvb Id Number, representing a Publisher.</param>
        /// <returns>A JsonDataContainerPublisher, containing the Publisher Data.</returns>
        public JsonDataContainerPublisher getPublisherData(string accessToken, string mvbId)
        {
            //Check Parameters
            if (string.IsNullOrWhiteSpace(accessToken))
            {
                throw new Exception("AccessToken may not be null or empty!");
            }

            if (string.IsNullOrWhiteSpace(mvbId))
            {
                throw new Exception("MvbId may not be null or empty!");
            }

            //Variables
            RestClient client;
            HttpWebResponse json;
            string responseAsString = null;

            //Init new RestClient
            client = new RestClient();
            client.EndPoint = string.Format(endpoint + "publisher/{0}", mvbId);
            client.AccessToken = string.Format("Bearer {0}", accessToken);
            client.ContentTypeString = "application/json";
            client.Method = HttpVerb.GET;

            //Send Request
            try
            {
                json = client.MakeRequest("", ref responseAsString);
            }
            catch (Exception ex)
            {
                throw new Exception("An Error occured:\n\n" + ex.Message);
            }

            //Init Json.net Deserializer
            JsonSerializerSettings jsonSerializerSettings = new JsonSerializerSettings();
            jsonSerializerSettings.MissingMemberHandling = MissingMemberHandling.Ignore;
            jsonSerializerSettings.NullValueHandling = NullValueHandling.Ignore;

            //Create a Container, containing all Json Fields and deserialize Json Data
            return JsonConvert.DeserializeObject<JsonDataContainerPublisher>(responseAsString, jsonSerializerSettings);
        }



        /// <summary>
        /// Requests an Index Search and returns the Results as a JsonContainer Array.
        /// </summary>
        /// <param name="accessToken">A statefull Access Token generated by the login-Method or a fixed stateless Access Token.</param>
        /// <param name="index">Index to search for.</param>
        /// <param name="searchQuery">The Search Query to search the selected Index for.</param>
        /// <returns>A JsonDataContainerIndex Array, containing the Search Results for the Index Search.</returns>
        public JsonDataContainerIndex[] getIndexData(string accessToken, index index, string searchQuery)
        {
            //Check Parameters
            if (string.IsNullOrWhiteSpace(accessToken))
            {
                throw new Exception("AccessToken may not be null or empty!");
            }

            if (string.IsNullOrWhiteSpace(searchQuery))
            {
                throw new Exception("SearchQuery may not be null or empty!");
            }

            //Variables
            RestClient client;
            HttpWebResponse json;
            string responseAsString = null;

            //Init new RestClient
            client = new RestClient();
            client.EndPoint = endpoint + "index/" + index.ToString() + @"/" + searchQuery;
            client.AccessToken = string.Format("Bearer {0}", accessToken);
            client.ContentTypeString = "application/json";
            client.Method = HttpVerb.GET;

            //Send Request
            try
            {
                json = client.MakeRequest("", ref responseAsString);
            }
            catch (Exception ex)
            {
                throw new Exception("An Error occured:\n\n" + ex.Message);
            }

            //Init Json.net Deserializer
            JsonSerializerSettings jsonSerializerSettings = new JsonSerializerSettings();
            jsonSerializerSettings.MissingMemberHandling = MissingMemberHandling.Ignore;
            jsonSerializerSettings.NullValueHandling = NullValueHandling.Ignore;

            //Create a Container, containing all Json Fields and deserialize Json Data
            return JsonConvert.DeserializeObject<JsonDataContainerIndex[]>(responseAsString, jsonSerializerSettings);
        }



        /// <summary>
        /// Returns an Array of JsonContainer, containing all available Media Data URLs for the specified Product.
        /// </summary>
        /// <param name="mediaToken">A statefull Access Token generated by the login-Method or a fixed stateless Media Token.</param>
        /// <param name="Id">The unique VLB Product Id (not the ISBN). You can get the Product's Id via the getTitleDetailData-Method.</param>
        /// <returns>A JsonDataContainerMedia Array, containing all Media URLs for the specified Product.</returns>
        public JsonDataContainerMedia[] getMediaData(string mediaToken, string Id)
        {
            //Check Parameters
            if (string.IsNullOrWhiteSpace(mediaToken))
            {
                throw new Exception("AccessToken may not be null or empty!");
            }

            if (string.IsNullOrWhiteSpace(Id))
            {
                throw new Exception("Id may not be null or empty!");
            }

            //Variables
            RestClient client;
            HttpWebResponse json;
            string responseAsString = null;

            //Init new RestClient
            client = new RestClient();
            client.EndPoint = endpoint + "asset/mmo/" + Id;
            client.AccessToken = string.Format("Bearer {0}", mediaToken);
            client.ContentTypeString = "application/json";
            client.Method = HttpVerb.GET;

            //Send Request
            try
            {
                json = client.MakeRequest("", ref responseAsString);
            }
            catch (Exception ex)
            {
                throw new Exception("An Error occured:\n\n" + ex.Message);
            }

            //Init Json.net Deserializer
            JsonSerializerSettings jsonSerializerSettings = new JsonSerializerSettings();
            jsonSerializerSettings.MissingMemberHandling = MissingMemberHandling.Ignore;
            jsonSerializerSettings.NullValueHandling = NullValueHandling.Ignore;

            //Create a Container, containing all Json Fields and deserialize Json Data
            return JsonConvert.DeserializeObject<JsonDataContainerMedia[]>(responseAsString, jsonSerializerSettings);
        }



        /// <summary>
        /// Downloads a Title Cover with the specified Size.
        /// </summary>
        /// <param name="coverToken">A statefull Access Token generated by the login-Method or a fixed stateless Cover Token.</param>
        /// <param name="isbn">The unique 13-digit Identifier, with or without Hyphen, e.g. "123-4-56789-0123" or "1234567890123".</param>
        /// <param name="coversize">The desired Cover Size. Refer to the VLB Rest api Manual for more Information.</param>
        /// <returns>The Cover as Image, which can be used directly in your UI or saved onto a local Drive.</returns>
        public Image downloadCover(string coverToken, string isbn, coverSize coversize = coverSize.m)
        {
            //Check Parameters
            if (string.IsNullOrWhiteSpace(coverToken))
            {
                throw new Exception("AccessToken may not be null or empty!");
            }

            if (string.IsNullOrWhiteSpace(isbn))
            {
                throw new Exception("ISBN may not be null or empty!");
            }

            //Variables
            RestClient client;
            HttpWebResponse httpWebResponse;
            Stream stream;
            
            //Init new RestClient
            client = new RestClient();
            client.EndPoint = coversize != coverSize.original ? (endpoint + "cover/" + isbn.Replace("-", "").Trim() + @"/" + coversize.ToString()) : (endpoint + "cover/" + isbn.Replace("-", "").Trim());
            client.AccessToken = string.Format("Bearer {0}", coverToken);
            client.ContentTypeString = "application/json";
            client.Method = HttpVerb.GET;

            //Send Request and get Image
            try
            {
                httpWebResponse = client.MakeRequestReturnResponse("");
                stream = httpWebResponse.GetResponseStream();

                return Image.FromStream(stream);
            }
            catch (Exception ex)
            {
                throw new Exception("An Error occured:\n\n" + ex.Message);
            }
        }



        /// <summary>
        /// Downloads the specified Media Data and redirects the Data to the provided Stream.
        /// </summary>
        /// <param name="mediaToken">A statefull Access Token generated by the login-Method or a fixed stateless Media Token.</param>
        /// <param name="jsonDataContainerMedia">A JsonDataContainer, containing an URL to the desired Media Data.</param>
        /// <param name="destination">A Stream to copy the Data into.</param>
        /// <returns>The File-Format as string of the downloaded Media Data, so you may determine what to do with the provided Stream next. If the File-Format is unknown, the Header-Information will be attached, so you may try to determine the File-Format yourself.</returns>
        public string downloadMedia(string mediaToken, JsonDataContainerMedia jsonDataContainerMedia, ref Stream destination)
        {
            //Check Parameters
            if (string.IsNullOrWhiteSpace(mediaToken))
            {
                throw new Exception("AccessToken may not be null or empty!");
            }

            if (jsonDataContainerMedia == null)
            {
                throw new Exception("JsonDataContainerMedia may not be null!");
            }

            if (destination == null)
            {
                throw new Exception("Destination-Stream may not be null!");
            }

            //Variables
            RestClient client;
            HttpWebResponse httpWebResponse;
            Stream stream;
            string contentTypeHeader = "";
            CultureInfo cultureInfo = CultureInfo.CurrentCulture;

            //Init new RestClient
            client = new RestClient();
            client.EndPoint = jsonDataContainerMedia.url;
            client.AccessToken = string.Format("Bearer {0}", mediaToken);
            client.ContentTypeString = "application/json";
            client.Method = HttpVerb.GET;

            //Send Request and get Image, if possible
            try
            {
                //Init Stream, get Header Information about File Format
                httpWebResponse = client.MakeRequestReturnResponse("");
                stream = httpWebResponse.GetResponseStream();

                //Write to Stream
                stream.CopyTo(destination);
                stream.Flush();
                stream.Close();

                //Guess FileFormat
                contentTypeHeader = httpWebResponse.Headers["Content-Type"];

                if (cultureInfo.CompareInfo.IndexOf(contentTypeHeader, "image/jpeg", CompareOptions.IgnoreCase) >= 0)
                {
                    return "jpg";
                }

                if (cultureInfo.CompareInfo.IndexOf(contentTypeHeader, "application/pdf", CompareOptions.IgnoreCase) >= 0)
                {
                    return "pdf";
                }

                if (cultureInfo.CompareInfo.IndexOf(contentTypeHeader, "video/x-msvideo", CompareOptions.IgnoreCase) >= 0)
                {
                    return "avi";
                }

                if (cultureInfo.CompareInfo.IndexOf(contentTypeHeader, "audio/mpeg", CompareOptions.IgnoreCase) >= 0)
                {
                    return "mp3";
                }

                if (cultureInfo.CompareInfo.IndexOf(contentTypeHeader, "application/epub+zip", CompareOptions.IgnoreCase) >= 0)
                {
                    return "zip";
                }

                return "unknown: " + contentTypeHeader;
            }
            catch (Exception ex)
            {
                throw new Exception("An Error occured:\n\n" + ex.Message);
            }
        }
    }
}
